@@ -11,10 +11,12 @@
 //
 //===----------------------------------------------------------------------===//
 
-/* Capstone Disassembler Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013> */
+/* Capstone Disassembly Engine */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
 
-#include <inttypes.h>
+#ifdef CAPSTONE_HAS_MIPS
+
+#include "../../inttypes.h"
 #include <stdlib.h>
 #include <stdio.h>	// debug
 #include <string.h>
@@ -91,12 +93,12 @@ static void set_mem_access(MCInst *MI, bool status)
 		return;
 
 	if (status) {
-		MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].type = MIPS_OP_MEM;
-		MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].mem.base = MIPS_REG_INVALID;
-		MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].mem.disp = 0;
+		MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].type = MIPS_OP_MEM;
+		MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].mem.base = MIPS_REG_INVALID;
+		MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].mem.disp = 0;
 	} else {
 		// done, create the next operand slot
-		MI->flat_insn.mips.op_count++;
+		MI->flat_insn->detail->mips.op_count++;
 	}
 }
 
@@ -150,49 +152,16 @@ static void printRegName(SStream *OS, unsigned RegNo)
 	SStream_concat(OS, "$%s", getRegisterName(RegNo));
 }
 
-static void printSaveRestore(MCInst *MI, SStream *O)
-{
-	unsigned i, e;
-	for (i = 0, e = MCInst_getNumOperands(MI); i != e; ++i) {
-		if (i != 0)
-			SStream_concat(O, ", ");
-		if (MCOperand_isReg(MCInst_getOperand(MI, i)))
-			printRegName(O, MCOperand_getReg(MCInst_getOperand(MI, i)));
-		else
-			printUnsignedImm(MI, i, O);
-	}
-}
-
 void Mips_printInst(MCInst *MI, SStream *O, void *info)
 {
 	char *mnem;
 
 	switch (MCInst_getOpcode(MI)) {
 		default: break;
-		case Mips_RDHWR:
-		case Mips_RDHWR64:
-			SStream_concat(O, ".set\tpush\n");
-			SStream_concat(O, ".set\tmips32r2\n");
-			break;
 		case Mips_Save16:
-			SStream_concat(O, "\tsave\t");
-			printSaveRestore(MI, O);
-			SStream_concat(O, " # 16 bit inst\n");
-			return;
 		case Mips_SaveX16:
-			SStream_concat(O, "\tsave\t");
-			printSaveRestore(MI, O);
-			SStream_concat(O, "\n");
-			return;
 		case Mips_Restore16:
-			SStream_concat(O, "\trestore\t");
-			printSaveRestore(MI, O);
-			SStream_concat(O, " # 16 bit inst\n");
-			return;
 		case Mips_RestoreX16:
-			SStream_concat(O, "\trestore\t");
-			printSaveRestore(MI, O);
-			SStream_concat(O, "\n");
 			return;
 	}
 
@@ -200,8 +169,9 @@ void Mips_printInst(MCInst *MI, SStream *O, void *info)
 	mnem = printAliasInstr(MI, O, info);
 	if (!mnem) {
 		mnem = printAlias(MI, O);
-		if (!mnem)
+		if (!mnem) {
 			printInstruction(MI, O, NULL);
+		}
 	}
 
 	if (mnem) {
@@ -209,35 +179,30 @@ void Mips_printInst(MCInst *MI, SStream *O, void *info)
 		MCInst_setOpcodePub(MI, Mips_map_insn(mnem));
 		cs_mem_free(mnem);
 	}
-
-	switch (MCInst_getOpcode(MI)) {
-		default: break;
-		case Mips_RDHWR:
-		case Mips_RDHWR64:
-			SStream_concat(O, "\n.set\tpop");
-			break;
-	}
 }
 
 static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 {
-	MCOperand *Op = MCInst_getOperand(MI, OpNo);
+	MCOperand *Op;
+
+	if (OpNo >= MI->size)
+		return;
+
+	Op = MCInst_getOperand(MI, OpNo);
 	if (MCOperand_isReg(Op)) {
 		unsigned int reg = MCOperand_getReg(Op);
 		printRegName(O, reg);
 		reg = Mips_map_register(reg);
 		if (MI->csh->detail) {
 			if (MI->csh->doing_mem) {
-				MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].mem.base = reg;
+				MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].mem.base = reg;
 			} else {
-				MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].type = MIPS_OP_REG;
-				MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].reg = reg;
-				MI->flat_insn.mips.op_count++;
+				MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].type = MIPS_OP_REG;
+				MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].reg = reg;
+				MI->flat_insn->detail->mips.op_count++;
 			}
 		}
-	}
-
-	if (MCOperand_isImm(Op)) {
+	} else if (MCOperand_isImm(Op)) {
 		int64_t imm = MCOperand_getImm(Op);
 		if (MI->csh->doing_mem) {
 			if (imm) {	// only print Imm offset if it is not 0
@@ -247,14 +212,14 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 					else
 						SStream_concat(O, "%"PRIu64, imm);
 				} else {
-					if (imm <= -HEX_THRESHOLD)
+					if (imm < -HEX_THRESHOLD)
 						SStream_concat(O, "-0x%"PRIx64, -imm);
 					else
 						SStream_concat(O, "-%"PRIu64, -imm);
 				}
 			}
 			if (MI->csh->detail)
-				MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].mem.disp = imm;
+				MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].mem.disp = imm;
 		} else {
 			if (imm >= 0) {
 				if (imm > HEX_THRESHOLD)
@@ -262,16 +227,16 @@ static void printOperand(MCInst *MI, unsigned OpNo, SStream *O)
 				else
 					SStream_concat(O, "%"PRIu64, imm);
 			} else {
-				if (imm <= -HEX_THRESHOLD)
+				if (imm < -HEX_THRESHOLD)
 					SStream_concat(O, "-0x%"PRIx64, -imm);
 				else
 					SStream_concat(O, "-%"PRIu64, -imm);
 			}
 
 			if (MI->csh->detail) {
-				MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].type = MIPS_OP_IMM;
-				MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].imm = imm;
-				MI->flat_insn.mips.op_count++;
+				MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].type = MIPS_OP_IMM;
+				MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].imm = imm;
+				MI->flat_insn->detail->mips.op_count++;
 			}
 		}
 	}
@@ -288,15 +253,15 @@ static void printUnsignedImm(MCInst *MI, int opNum, SStream *O)
 			else
 				SStream_concat(O, "%u", (unsigned short int)imm);
 		} else {
-			if (imm <= -HEX_THRESHOLD)
+			if (imm < -HEX_THRESHOLD)
 				SStream_concat(O, "-0x%x", (short int)-imm);
 			else
 				SStream_concat(O, "-%u", (short int)-imm);
 		}
 		if (MI->csh->detail) {
-			MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].type = MIPS_OP_IMM;
-			MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].imm = (unsigned short int)imm;
-			MI->flat_insn.mips.op_count++;
+			MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].type = MIPS_OP_IMM;
+			MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].imm = (unsigned short int)imm;
+			MI->flat_insn->detail->mips.op_count++;
 		}
 	} else
 		printOperand(MI, opNum, O);
@@ -312,9 +277,9 @@ static void printUnsignedImm8(MCInst *MI, int opNum, SStream *O)
 		else
 			SStream_concat(O, "%u", imm);
 		if (MI->csh->detail) {
-			MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].type = MIPS_OP_IMM;
-			MI->flat_insn.mips.operands[MI->flat_insn.mips.op_count].imm = imm;
-			MI->flat_insn.mips.op_count++;
+			MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].type = MIPS_OP_IMM;
+			MI->flat_insn->detail->mips.operands[MI->flat_insn->detail->mips.op_count].imm = imm;
+			MI->flat_insn->detail->mips.op_count++;
 		}
 	} else
 		printOperand(MI, opNum, O);
@@ -327,9 +292,9 @@ static void printMemOperand(MCInst *MI, int opNum, SStream *O)
 	// pattern lw $25,%call16($28)
 	set_mem_access(MI, true);
 	printOperand(MI, opNum + 1, O);
-	SStream_concat(O, "(");
+	SStream_concat0(O, "(");
 	printOperand(MI, opNum, O);
-	SStream_concat(O, ")");
+	SStream_concat0(O, ")");
 	set_mem_access(MI, false);
 }
 
@@ -339,7 +304,7 @@ static void printMemOperandEA(MCInst *MI, int opNum, SStream *O)
 	// when using stack locations for not load/store instructions
 	// print the same way as all normal 3 operand instructions.
 	printOperand(MI, opNum, O);
-	SStream_concat(O, ", ");
+	SStream_concat0(O, ", ");
 	printOperand(MI, opNum + 1, O);
 	return;
 }
@@ -347,7 +312,7 @@ static void printMemOperandEA(MCInst *MI, int opNum, SStream *O)
 static void printFCCOperand(MCInst *MI, int opNum, SStream *O)
 {
 	MCOperand *MO = MCInst_getOperand(MI, opNum);
-	SStream_concat(O, MipsFCCToString((Mips_CondCode)MCOperand_getImm(MO)));
+	SStream_concat0(O, MipsFCCToString((Mips_CondCode)MCOperand_getImm(MO)));
 }
 
 static char *printAlias1(char *Str, MCInst *MI, unsigned OpNo, SStream *OS)
@@ -363,7 +328,7 @@ static char *printAlias2(char *Str, MCInst *MI,
 	char *tmp;
 
 	tmp = printAlias1(Str, MI, OpNo0, OS);
-	SStream_concat(OS, ", ");
+	SStream_concat0(OS, ", ");
 	printOperand(MI, OpNo1, OS);
 
 	return tmp;
@@ -383,6 +348,11 @@ static char *printAlias(MCInst *MI, SStream *OS)
 			if (isReg(MI, 1, Mips_ZERO))
 				return printAlias2("beqz", MI, 0, 2, OS);
 			return NULL;
+		case Mips_BEQL:
+			// beql $r0, $zero, $L2 => beqzl $r0, $L2
+			if (isReg(MI, 0, Mips_ZERO) && isReg(MI, 1, Mips_ZERO))
+				return printAlias2("beqzl", MI, 0, 2, OS);
+			return NULL;
 		case Mips_BEQ64:
 			// beq $r0, $zero, $L2 => beqz $r0, $L2
 			if (isReg(MI, 1, Mips_ZERO_64))
@@ -393,6 +363,11 @@ static char *printAlias(MCInst *MI, SStream *OS)
 			if (isReg(MI, 1, Mips_ZERO))
 				return printAlias2("bnez", MI, 0, 2, OS);
 			return NULL;
+		case Mips_BNEL:
+			// bnel $r0, $zero, $L2 => bnezl $r0, $L2
+			if (isReg(MI, 1, Mips_ZERO))
+				return printAlias2("bnezl", MI, 0, 2, OS);
+			return NULL;
 		case Mips_BNE64:
 			// bne $r0, $zero, $L2 => bnez $r0, $L2
 			if (isReg(MI, 1, Mips_ZERO_64))
@@ -446,3 +421,4 @@ static char *printAlias(MCInst *MI, SStream *OS)
 #define PRINT_ALIAS_INSTR
 #include "MipsGenAsmWriter.inc"
 
+#endif
