@@ -363,7 +363,7 @@ cs_err cs_option(csh ud, cs_opt_type type, size_t value)
 		default:
 			break;
 		case CS_OPT_DETAIL:
-			handle->detail = value;
+			handle->detail = (cs_opt_value)value;
 			return CS_ERR_OK;
 		case CS_OPT_SKIPDATA:
 			handle->skipdata = (value == CS_OPT_ON);
