@@ -24,12 +24,13 @@ struct platform {
 	int syntax;
 };
 
-static char* hex_string(unsigned char *str, int len)
+static char *hex_string(unsigned char *str, size_t len)
 {
 	// returns a malloced string that has the hex version of the string in it
 	// null if failed to malloc
-	char * hex_out = NULL;
-	int i = 0;
+	char *hex_out;
+	size_t i;
+
 	hex_out = (char *) malloc(len*2 + 1); // two ascii characters per input character, plus trailing null
 	if (!hex_out) { goto Exit; }
 
@@ -195,8 +196,8 @@ static void test_invalids()
 		cs_option(handle, CS_OPT_SYNTAX, CS_OPT_SYNTAX_NOREGNAME);
 
 		for (j = 0; j < invalid->num_invalid_codes; ++j) {
-			struct invalid_code * invalid_code = NULL;
-			char * hex_str = NULL;
+			struct invalid_code *invalid_code = NULL;
+			char *hex_str = NULL;
 
 			invalid_code = invalid->invalid_codes + j;
 
@@ -234,7 +235,7 @@ struct valid_code {
 	unsigned char *code;
 	size_t size;
 	uint32_t start_addr;
-	char* expected_out;
+	char *expected_out;
 	char *comment;
 };
 
